@@ -60,17 +60,13 @@ static const char *menu_file_paths[] = { "./", "~/.camv-rnd/", NULL, NULL };
 static const char *menu_name_fmt = "menu.lht";
 
 #define CONF_USER_DIR "~/.camv-rnd"
+
 const char *rnd_conf_userdir_path = CONF_USER_DIR;
 const char *rnd_conf_user_path = CONF_USER_DIR "/conf_core.lht";
 
 
 extern void camv_main_act_init2(void);
 
-void rnd_tool_gui_init(void)
-{
-
-}
-
 static void gui_support_plugins(int load)
 {
 	static int loaded = 0;
@@ -143,6 +139,8 @@ int main(int argc, char *argv[])
 	rnd_app.expose_main = camv_expose_main;
 	rnd_app.expose_preview = camv_expose_preview;
 
+	rnd_app.dot_dir = CONF_USER_DIR;
+	rnd_app.lib_dir = LIBDIR;
 
 	rnd_fix_locale_and_env();
 
