@@ -162,7 +162,7 @@
 					break;\
 			} \
 			else if (__c__ == '+') { \
-				if (__i__ != 0); \
+				if (__i__ != 0) \
 					break;\
 			} \
 			else if (_permit_decpt_ && (__c__ == '.')) { \
@@ -350,7 +350,7 @@ static ge_parse_res_t geparse_aperture_spec_shaped(geparse_ctx_t *ctx, long aid,
 			inst->data.aper.shape = GEA_CIRC;
 			READ_COORD_DEC(ctx, inst->data.aper.data.circ.dia);
 			READ(ctx, c);
-			if (c == '*')
+			if (c == '*') /* it seems a few arguments at the end are optional; especially rotation for '1' (circle) may be missing */
 				break;
 			if (c != 'X') {
 				ctx->errmsg = "unexpected character in circle aperture def";
@@ -511,6 +511,8 @@ static int geparse_macro_aperture_line(geparse_ctx_t *ctx, ge_aper_macro_t *am,
 	for(a = 0; a < argc; a++) {
 		ge_expr_prg_t *e;
 		READ(ctx, c);
+		if (c == '*')
+			return 0;
 		if ((a < argc-1) && (c != ',')) {
 			ctx->errmsg = "macro aperture: missing comma (not enough parameters?)";
 			return -1;
@@ -862,6 +864,13 @@ ge_parse_res_t geparse_long_cmd(geparse_ctx_t *ctx)
 			READ_TILL_CMDEND(ctx);
 			break;
 
+		/* these are just overly fancy comments really -> ignore */
+		case CMD('T', 'F'):
+		case CMD('T', 'A'):
+		case CMD('T', 'D'):
+		case CMD('T', 'O'):
+			READ_TILL_CMDEND(ctx);
+			break;
 
 		default:
 			ctx->errmsg = "unrecognized long command";
