@@ -120,7 +120,7 @@ static camv_any_obj_t *polyarea2camv(rnd_polyarea_t *src)
 			grp->objs = malloc(sizeof(camv_any_obj_t) * grp->len);
 			pa = src;
 			do {
-				camv_poly_t *dst = &grp->objs[i];
+				camv_poly_t *dst = &grp->objs[i].poly;
 				camv_poly_init(dst);
 				pline2camv(dst, pa->contours);
 				i++;
@@ -141,18 +141,24 @@ static camv_any_obj_t *polyarea2camv(rnd_polyarea_t *src)
 /* Convert an oblong aperture into a camv polygon around 0;0 */
 static camv_any_obj_t *aper_oblong2poly(ge_aper_t *aper, double rot)
 {
-	rnd_coord_t xh, yh;
+	rnd_coord_t xs = aper->data.oblong.xs, ys = aper->data.oblong.ys, xh, yh;
 	rnd_polyarea_t *pa, *ph, *ptmp;
 	camv_any_obj_t *o;
 	int n;
 
 	TODO("figure if we need rotation");
-	xh = rnd_round((double)aper->data.oblong.xs / 2.0);
-	yh = rnd_round((double)aper->data.oblong.ys / 2.0);
+
+	/* real thin oblong is sometimes made to be 0 in size that would be a
+	   self intersecting poly; fix that up */
+	if (xs < 2) xs = 2;
+	if (ys < 2) ys = 2;
+
+	xh = rnd_round((double)xs / 2.0);
+	yh = rnd_round((double)ys / 2.0);
 	if (xh > yh)
-		pa = rnd_poly_from_line(-xh+yh, 0, +xh-yh, 0, aper->data.oblong.ys, 0);
+		pa = rnd_poly_from_line(-xh+yh, 0, +xh-yh, 0, ys, 0);
 	else
-		pa = rnd_poly_from_line(0, -yh+xh, 0, +yh-xh, aper->data.oblong.xs, 0);
+		pa = rnd_poly_from_line(0, -yh+xh, 0, +yh-xh, xs, 0);
 
 	/* subtract the hole if there's one */
 	if (aper->hole > 0) {
@@ -222,7 +228,7 @@ static int param_eval_int(const ge_macro_line_t *l, int paridx, vtd0_t *param)
 {
 	double res;
 	ge_expr_prg_t **prg = (ge_expr_prg_t **)vtp0_get((vtp0_t *)&l->operand, paridx-1, 0);
-	assert(prg != NULL); /* the parser should have checked argc already */
+	if (prg == NULL) return 0;
 	gex_eval(*prg, param, &res);
 	return rnd_round(res);
 }
@@ -232,19 +238,23 @@ static double param_eval_dbl(const ge_macro_line_t *l, int paridx, vtd0_t *param
 {
 	double res;
 	ge_expr_prg_t **prg = (ge_expr_prg_t **)vtp0_get((vtp0_t *)&l->operand, paridx-1, 0);
-	assert(prg != NULL); /* the parser should have checked argc already */
+	if (prg == NULL) return 0;
 	gex_eval(*prg, param, &res);
 	return res;
 }
 
 /* idx is counted from 1 for code clarity */
-static rnd_coord_t param_eval_crd(const ge_macro_line_t *l, int paridx, vtd0_t *param)
+static rnd_coord_t param_eval_crd(gedraw_ctx_t *ctx, const ge_macro_line_t *l, int paridx, vtd0_t *param)
 {
 	double res;
 	ge_expr_prg_t **prg = (ge_expr_prg_t **)vtp0_get((vtp0_t *)&l->operand, paridx-1, 0);
-	assert(prg != NULL); /* the parser should have checked argc already */
+	if (prg == NULL) return 0;
 	gex_eval(*prg, param, &res);
-TODO("respect ctx unit");
+
+	if (ctx->aper_unit == GEU_INCH)
+		res *= 25.4;
+	/* else we are in mm already */
+
 	return (rnd_coord_t)RND_MM_TO_COORD(res);
 }
 
@@ -331,7 +341,7 @@ static rnd_polyarea_t *pa_ring(rnd_coord_t cx, rnd_coord_t cy, rnd_coord_t d_out
 }
 
 /* Render a single line (primitive) of a macro into an accumulating polyarea dst */
-static int macro_line_render(rnd_polyarea_t **dst, const ge_macro_line_t *l, vtd0_t *param, double rot)
+static int macro_line_render(gedraw_ctx_t *ctx, rnd_polyarea_t **dst, const ge_macro_line_t *l, vtd0_t *param, double rot)
 {
 	int pol, corners, n, maxrng;
 	rnd_coord_t x, y, d, th, x1, y1, x2, y2, od, id, rth, chth, chd, gap, w, h;
@@ -343,9 +353,9 @@ static int macro_line_render(rnd_polyarea_t **dst, const ge_macro_line_t *l, vtd
 	switch(l->op) {
 		case GEMO_CIRC:
 			pol = param_eval_int(l, 1, param);
-			d = param_eval_crd(l, 2, param);
-			x = param_eval_crd(l, 3, param);
-			y = param_eval_crd(l, 4, param);
+			d = param_eval_crd(ctx, l, 2, param);
+			x = param_eval_crd(ctx, l, 3, param);
+			y = param_eval_crd(ctx, l, 4, param);
 			r = param_eval_dbl(l, 5, param) + rot;
 			if (r != 0)
 				rnd_rotate(&x, &y, 0, 0, cos(-r / RND_RAD_TO_DEG), sin(-r / RND_RAD_TO_DEG));
@@ -358,13 +368,13 @@ static int macro_line_render(rnd_polyarea_t **dst, const ge_macro_line_t *l, vtd
 			r = param_eval_dbl(l, l->operand.used, param) + rot;
 			cs = cos(-r / RND_RAD_TO_DEG);
 			sn = sin(-r / RND_RAD_TO_DEG);
-			v[0] = param_eval_crd(l, 3, param);
-			v[1] = param_eval_crd(l, 4, param);
+			v[0] = param_eval_crd(ctx, l, 3, param);
+			v[1] = param_eval_crd(ctx, l, 4, param);
 			rnd_rotate(&v[0], &v[1], 0, 0, cs, sn);
 			contour = rnd_poly_contour_new(v);
 			for(n = 1; n < corners; n++) {
-				v[0] = param_eval_crd(l, n*2+3, param);
-				v[1] = param_eval_crd(l, n*2+4, param);
+				v[0] = param_eval_crd(ctx, l, n*2+3, param);
+				v[1] = param_eval_crd(ctx, l, n*2+4, param);
 				rnd_rotate(&v[0], &v[1], 0, 0, cs, sn);
 				rnd_poly_vertex_include(contour->head->prev, rnd_poly_node_create(v));
 			}
@@ -374,9 +384,9 @@ static int macro_line_render(rnd_polyarea_t **dst, const ge_macro_line_t *l, vtd
 		case GEMO_REGPOLY:
 			pol = param_eval_int(l, 1, param);
 			corners = param_eval_int(l, 2, param);
-			x = param_eval_crd(l, 3, param);
-			y = param_eval_crd(l, 4, param);
-			d = param_eval_crd(l, 5, param);
+			x = param_eval_crd(ctx, l, 3, param);
+			y = param_eval_crd(ctx, l, 4, param);
+			d = param_eval_crd(ctx, l, 5, param);
 			r = param_eval_dbl(l, 6, param) + rot;
 			if (r != 0)
 				rnd_rotate(&x, &y, 0, 0, cos(-r / RND_RAD_TO_DEG), sin(-r / RND_RAD_TO_DEG));
@@ -384,14 +394,14 @@ static int macro_line_render(rnd_polyarea_t **dst, const ge_macro_line_t *l, vtd
 			macro_line_render_merge(dst, pb, pol);
 			return 0;
 		case GEMO_MOIRE:
-			x = x1 = param_eval_crd(l, 1, param);
-			y = y1 = param_eval_crd(l, 2, param);
-			od = param_eval_crd(l, 3, param);
-			rth = param_eval_crd(l, 4, param);
-			gap = param_eval_crd(l, 5, param);
+			x = x1 = param_eval_crd(ctx, l, 1, param);
+			y = y1 = param_eval_crd(ctx, l, 2, param);
+			od = param_eval_crd(ctx, l, 3, param);
+			rth = param_eval_crd(ctx, l, 4, param);
+			gap = param_eval_crd(ctx, l, 5, param);
 			maxrng = param_eval_int(l, 6, param);
-			chth = param_eval_crd(l, 7, param);
-			chd = param_eval_crd(l, 8, param);
+			chth = param_eval_crd(ctx, l, 7, param);
+			chd = param_eval_crd(ctx, l, 8, param);
 			r = param_eval_dbl(l, 9, param) + rot;
 			cs = cos(-r / RND_RAD_TO_DEG);
 			sn = sin(-r / RND_RAD_TO_DEG);
@@ -419,11 +429,11 @@ static int macro_line_render(rnd_polyarea_t **dst, const ge_macro_line_t *l, vtd
 			macro_line_render_merge(dst, pd, 1);
 			return 0;
 		case GEMO_THERM:
-			x = x1 = param_eval_crd(l, 1, param);
-			y = y1 = param_eval_crd(l, 2, param);
-			od = param_eval_crd(l, 3, param);
-			id = param_eval_crd(l, 4, param);
-			th = param_eval_crd(l, 5, param);
+			x = x1 = param_eval_crd(ctx, l, 1, param);
+			y = y1 = param_eval_crd(ctx, l, 2, param);
+			od = param_eval_crd(ctx, l, 3, param);
+			id = param_eval_crd(ctx, l, 4, param);
+			th = param_eval_crd(ctx, l, 5, param);
 			r = param_eval_dbl(l, 6, param) + rot;
 
 			cs = cos(-r / RND_RAD_TO_DEG);
@@ -446,11 +456,11 @@ static int macro_line_render(rnd_polyarea_t **dst, const ge_macro_line_t *l, vtd
 			return 0;
 		case GEMO_LINE_XY:
 			pol = param_eval_int(l, 1, param);
-			th = param_eval_crd(l, 2, param);
-			x1 = param_eval_crd(l, 3, param);
-			y1 = param_eval_crd(l, 4, param);
-			x2 = param_eval_crd(l, 5, param);
-			y2 = param_eval_crd(l, 6, param);
+			th = param_eval_crd(ctx, l, 2, param);
+			x1 = param_eval_crd(ctx, l, 3, param);
+			y1 = param_eval_crd(ctx, l, 4, param);
+			x2 = param_eval_crd(ctx, l, 5, param);
+			y2 = param_eval_crd(ctx, l, 6, param);
 			r = param_eval_dbl(l, 7, param) + rot;
 
 			line_xy:;
@@ -468,7 +478,6 @@ static int macro_line_render(rnd_polyarea_t **dst, const ge_macro_line_t *l, vtd
 				len = 1;
 			nx /= len;
 			ny /= len;
-
 			pb = pa_rect(
 				rnd_round(x1 - nx*th/2.0), rnd_round(y1 - ny*th/2.0),
 				rnd_round(x2 - nx*th/2.0), rnd_round(y2 - ny*th/2.0),
@@ -478,10 +487,10 @@ static int macro_line_render(rnd_polyarea_t **dst, const ge_macro_line_t *l, vtd
 			return 0;
 		case GEMO_LINE_WH:
 			pol = param_eval_int(l, 1, param);
-			w = param_eval_crd(l, 2, param);
-			h = param_eval_crd(l, 3, param);
-			x = param_eval_crd(l, 4, param);
-			y = param_eval_crd(l, 5, param);
+			w = param_eval_crd(ctx, l, 2, param);
+			h = param_eval_crd(ctx, l, 3, param);
+			x = param_eval_crd(ctx, l, 4, param);
+			y = param_eval_crd(ctx, l, 5, param);
 			r = param_eval_dbl(l, 6, param) + rot;
 			x1 = x - w/2; x2 = x + w/2;
 			y1 = y2 = y;
@@ -498,7 +507,7 @@ static int macro_line_render(rnd_polyarea_t **dst, const ge_macro_line_t *l, vtd
 }
 
 /* Render a macro aperture using a macro def into a polygon */
-static camv_any_obj_t *aper_macro_render(ge_aper_t *aper, double rot)
+static camv_any_obj_t *aper_macro_render(ge_aper_t *aper, gedraw_ctx_t *ctx, double rot)
 {
 	rnd_polyarea_t *pa;
 	const ge_aper_macro_t *am = aper->data.macro.am;
@@ -511,7 +520,7 @@ static camv_any_obj_t *aper_macro_render(ge_aper_t *aper, double rot)
 
 	pa = NULL;
 	for(l = am->line1; l != NULL; l = l->next)
-		macro_line_render(&pa, l, &param, rot);
+		macro_line_render(ctx, &pa, l, &param, rot);
 
 	if (pa == NULL) {
 		rnd_message(RND_MSG_WARNING, "aperture macro results in empty render\n");
@@ -521,7 +530,7 @@ static camv_any_obj_t *aper_macro_render(ge_aper_t *aper, double rot)
 }
 
 
-static camv_any_obj_t *gedraw_render_aper_nohole(camv_design_t *camv, ge_aper_t *aper, rnd_coord_t ox, rnd_coord_t oy, double rot)
+static camv_any_obj_t *gedraw_render_aper_nohole(camv_design_t *camv, gedraw_ctx_t *ctx, ge_aper_t *aper, rnd_coord_t ox, rnd_coord_t oy, double rot)
 {
 	camv_any_obj_t *o;
 	double rx, ry;
@@ -574,7 +583,7 @@ static camv_any_obj_t *gedraw_render_aper_nohole(camv_design_t *camv, ge_aper_t
 
 		case GEA_MACRO:
 			if (aper->cached == NULL)
-				aper->cached = aper_macro_render(aper, rot);
+				aper->cached = aper_macro_render(aper, ctx, rot);
 			if (aper->cached == NULL) /* aperture creation may fail on empty aper */
 				return NULL;
 			o = camv_obj_dup(aper->cached);
@@ -584,7 +593,7 @@ static camv_any_obj_t *gedraw_render_aper_nohole(camv_design_t *camv, ge_aper_t
 	return NULL;
 }
 
-static camv_any_obj_t *gedraw_render_aper_hole(camv_design_t *camv, ge_aper_t *aper, rnd_coord_t ox, rnd_coord_t oy, double rot)
+static camv_any_obj_t *gedraw_render_aper_hole(camv_design_t *camv, gedraw_ctx_t *ctx, ge_aper_t *aper, rnd_coord_t ox, rnd_coord_t oy, double rot)
 {
 	camv_any_obj_t *o;
 	rnd_polyarea_t *pdiff, *ps, *ph;
@@ -632,7 +641,7 @@ TODO("Generate and cache these and dup with offset instead of reconstructing all
 			return o;
 
 		case GEA_MACRO:
-			return gedraw_render_aper_nohole(camv, aper, ox, oy, rot);
+			return gedraw_render_aper_nohole(camv, ctx, aper, ox, oy, rot);
 	}
 
 	return NULL;
@@ -640,12 +649,12 @@ TODO("Generate and cache these and dup with offset instead of reconstructing all
 
 /* render an aperture at x;y into a camv drawing primitive (may be a grp).
    Rot is in deg in gerb notation; x;y are in gerber notation. */
-static camv_any_obj_t *gedraw_render_aper(camv_design_t *camv, ge_aper_t *aper, rnd_coord_t ox, rnd_coord_t oy, double rot)
+static camv_any_obj_t *gedraw_render_aper(camv_design_t *camv, gedraw_ctx_t *ctx, ge_aper_t *aper, rnd_coord_t ox, rnd_coord_t oy, double rot)
 {
 	if (aper->hole == 0)
-		return gedraw_render_aper_nohole(camv, aper, ox, oy, rot);
+		return gedraw_render_aper_nohole(camv, ctx, aper, ox, oy, rot);
 	else
-		return gedraw_render_aper_hole(camv, aper, ox, oy, rot);
+		return gedraw_render_aper_hole(camv, ctx, aper, ox, oy, rot);
 }
 
 
@@ -779,7 +788,7 @@ static rnd_polyarea_t *pa_rect_side(gerbst_t *c1, gerbst_t *c2, double dx1, doub
 static void gedraw_do_draw_line(camv_design_t *camv, gedraw_ctx_t *ctx, camv_layer_t *ly, gerbst_t *curr, gerbst_t *last)
 {
 	camv_any_obj_t *o;
-	ge_aper_t *aper = gedraw_aper_get(ctx, curr->aper);
+	ge_aper_t *aper;
 	rnd_polyarea_t *slice, *sum, *pres;
 	double rx, ry;
 
@@ -789,9 +798,11 @@ static void gedraw_do_draw_line(camv_design_t *camv, gedraw_ctx_t *ctx, camv_lay
 		return;
 	}
 
+	aper = gedraw_aper_get(ctx, curr->aper);
+
 	switch(aper->shape) {
 		case GEA_CIRC:
-			o = camv_line_new();
+			o = (camv_any_obj_t *)camv_line_new();
 			o->line.x1 = ctx->ox + last->x; o->line.y1 = ctx->oy + last->y;
 			o->line.x2 = ctx->ox + curr->x; o->line.y2 = ctx->oy + curr->y;
 			o->line.thick = aper->data.circ.dia;
@@ -851,14 +862,24 @@ static void gedraw_do_draw_line(camv_design_t *camv, gedraw_ctx_t *ctx, camv_lay
 static void gedraw_do_draw_arc(camv_design_t *camv, gedraw_ctx_t *ctx, camv_layer_t *ly, gerbst_t *curr, gerbst_t *last, int is_cw)
 {
 	camv_arc_t *arc;
-	ge_aper_t *aper = gedraw_aper_get(ctx, curr->aper);
+	ge_aper_t *aper;
 	rnd_coord_t cx, cy, r1, r2;
 	double ang1, ang2, delta;
 
-	if (aper->shape != GEA_CIRC) {
-		rnd_message(RND_MSG_ERROR, "gedraw_do: DRAW arc: circular interpolation is permitted only with fileld circle aperture\n");
-		return;
+	if (!curr->poly) {
+		aper = gedraw_aper_get(ctx, curr->aper);
+		if (aper == NULL) {
+			rnd_message(RND_MSG_ERROR, "gedraw_do: DRAW arc: invalid aperture %ld\n", curr->aper);
+			return;
+		}
+
+		if (aper->shape != GEA_CIRC) {
+			rnd_message(RND_MSG_ERROR, "gedraw_do: DRAW arc: circular interpolation is permitted only with filled circle aperture\n");
+			return;
+		}
 	}
+	else
+		aper = NULL; /* in poly mode no need to use the aperture */
 
 	if (curr->quadr == GEQ_SINGLE) {
 		rnd_coord_t i = curr->i, j = curr->j;
@@ -950,7 +971,7 @@ static void gedraw_do_draw_arc(camv_design_t *camv, gedraw_ctx_t *ctx, camv_laye
 
 	arc = camv_arc_new();
 	arc->cx = ctx->ox + cx; arc->cy = ctx->oy + cy; arc->r = r1;
-	arc->thick = aper->data.circ.dia;
+	arc->thick = aper == NULL ? 0 : aper->data.circ.dia;
 	arc->start = 180 - ang1;
 	arc->delta = -delta;
 	camv_obj_add_to_layer(ly, (camv_any_obj_t *)arc);
@@ -970,8 +991,11 @@ static void gedraw_do_move(camv_design_t *camv, gedraw_ctx_t *ctx, camv_layer_t
 	/* auto-close poly contour on _real_ move; a move that stats in place is not
 	   really a move and shouldn't break the contour - this is how the reference
 	   viewer v2018.11b works */
-	if ((curr->poly) && ((curr->x != last->x) || (curr->y != last->y))) {
-		gedraw_poly_close(camv, ctx, ly);
+	if ((curr->poly) &&  ((curr->x != last->x) || (curr->y != last->y))) {
+		if (ctx->contour.used > 2)
+			gedraw_poly_close(camv, ctx, ly);
+		else
+			ctx->contour.used = 0; /* do not throw warnings in gedraw_poly_close() but make sure the poly is clean */
 		return;
 	}
 	/* normal draw: do nothing, coords are getting updated on the curr-to-last copy */
@@ -984,7 +1008,7 @@ static void gedraw_do_flash(camv_design_t *camv, gedraw_ctx_t *ctx, camv_layer_t
 
 	if (aper == NULL)
 		return;
-	o = gedraw_render_aper(camv, aper, curr->x + ctx->ox, curr->y + ctx->oy, 0);
+	o = gedraw_render_aper(camv, ctx, aper, curr->x + ctx->ox, curr->y + ctx->oy, 0);
 	if (o != NULL)
 		camv_obj_add_to_layer(ly, o);
 	else
@@ -1016,6 +1040,8 @@ static int gedraw_camv_(camv_design_t *camv, gedraw_ctx_t *ctx, const camv_layer
 	for(n = *from; n < ctx->code.used; n++) {
 		gedraw_inst_t *i = &ctx->code.array[n]; /* need to retrieve it each iteration because n may jump after SR */
 		switch(i->cmd) {
+			case GEC_MACRO_DEF:  break; /* shouldn't happen */
+
 			case GEC_SET_X:      SETCRD(curr->x, i->data.coord); break;
 			case GEC_SET_Y:      SETCRD(curr->y, i->data.coord); break;
 			case GEC_SET_I:      SETCRD(curr->i, i->data.coord); break;
@@ -1064,7 +1090,7 @@ static int gedraw_camv_(camv_design_t *camv, gedraw_ctx_t *ctx, const camv_layer
 					was_clearing = ly->clearing;
 					vx = i->data.steprep.x;
 					vy = i->data.steprep.y;
-					if ((vx <= 1) && (vy <= 1)) {
+					if ((vx < 1) && (vy < 1)) {
 						n++;
 						goto quit_sr; /* a new SR terminates an already open one */
 					}
@@ -1154,6 +1180,7 @@ int camv_gerb_load(camv_design_t *camv, const char *fn, FILE *f)
 	else
 		ctx.draw.acceptable_error = RND_MM_TO_COORD((double)pow(0.1, ctx.cfmt_fra));
 
+	ctx.draw.aper_unit = ctx.unit;
 	retv = gedraw_camv(camv, &ctx.draw, fn);
 
 	if (ctx.draw.aper_inited) {
